@@ -10,14 +10,14 @@ use warnings;
 
 use base qw( Tangence::Stream );
 
-our $VERSION = '0.13';
+our $VERSION = '0.14';
 
 use Carp;
 
 use Tangence::Constants;
 use Tangence::ObjectProxy;
 
-use constant VERSION_MINOR_MIN => 0;
+use constant VERSION_MINOR_MIN => 1;
 
 =head1 NAME
 
@@ -131,10 +131,8 @@ It takes the following named arguments:
 
 =item do_init => BOOL
 
-Optional. If true, sends the C<MSG_INIT> message first, to negotiate protocol
-version number. This will be performed by default in a future version, but is
-left optional for now, to accomodate servers that do not yet recognise
-C<MSG_INIT>.
+Ignored. Maintained for compatibility with previous version that allowed this
+to be disabled.
 
 =item on_root => CODE
 
@@ -159,39 +157,32 @@ sub tangence_connected
    my $self = shift;
    my %args = @_;
 
-   # Don't yet do MSG_INIT by default to support version -1 servers; but allow
-   # it optionally if requested
-   if( $args{do_init} ) {
-      $self->request(
-         request => Tangence::Message->new( $self, MSG_INIT )
-            ->pack_int( VERSION_MAJOR )
-            ->pack_int( VERSION_MINOR )
-            ->pack_int( VERSION_MINOR_MIN ),
+   $self->request(
+      request => Tangence::Message->new( $self, MSG_INIT )
+         ->pack_int( VERSION_MAJOR )
+         ->pack_int( VERSION_MINOR )
+         ->pack_int( VERSION_MINOR_MIN ),
 
-         on_response => sub {
-            my ( $message ) = @_;
-            my $type = $message->type;
+      on_response => sub {
+         my ( $message ) = @_;
+         my $type = $message->type;
 
-            if( $type == MSG_INITED ) {
-               my $major = $message->unpack_int();
-               my $minor = $message->unpack_int();
+         if( $type == MSG_INITED ) {
+            my $major = $message->unpack_int();
+            my $minor = $message->unpack_int();
 
-               $self->minor_version( $minor );
-               $self->tangence_initialised( %args );
-            }
-            elsif( $type == MSG_ERROR ) {
-               my $msg = $message->unpack_str();
-               print STDERR "Cannot initialise stream - error $msg";
-            }
-            else {
-               print STDERR "Cannot initialise stream - code $type\n";
-            }
-         },
-      );
-   }
-   else {
-      $self->tangence_initialised( %args );
-   }
+            $self->minor_version( $minor );
+            $self->tangence_initialised( %args );
+         }
+         elsif( $type == MSG_ERROR ) {
+            my $msg = $message->unpack_str();
+            print STDERR "Cannot initialise stream - error $msg";
+         }
+         else {
+            print STDERR "Cannot initialise stream - code $type\n";
+         }
+      },
+   );
 }
 
 sub tangence_initialised
