@@ -4,11 +4,8 @@ use strict;
 use warnings;
 use base qw( Tangence::Meta::Struct );
 
-our $VERSION = '0.20_001';
-
 use Carp;
 
-use Tangence::Type;
 use Tangence::Meta::Field;
 
 our %STRUCTS_BY_NAME;
@@ -26,7 +23,7 @@ sub new
 sub _new_type
 {
    my ( $sig ) = @_;
-   return Tangence::Type->new_from_sig( $sig );
+   return Tangence::Meta::Type->new_from_sig( $sig );
 }
 
 sub declare
@@ -41,7 +38,7 @@ sub declare
    for( $_ = 0; $_ < @{$args{fields}}; $_ += 2 ) {
       push @fields, Tangence::Meta::Field->new(
          name => $args{fields}[$_],
-         type => Tangence::Type->new_from_sig( $args{fields}[$_+1] ),
+         type => Tangence::Meta::Type->new_from_sig( $args{fields}[$_+1] ),
       );
    }
 
@@ -98,7 +95,7 @@ sub for_name
    my $class = shift;
    my ( $name ) = @_;
 
-   return $STRUCTS_BY_NAME{$name} || croak "Unknown Tangence::Struct for '$name'";
+   return $STRUCTS_BY_NAME{$name} or croak "Unknown Tangence::Struct for '$name'";
 }
 
 sub for_perlname
@@ -106,7 +103,7 @@ sub for_perlname
    my $class = shift;
    my ( $perlname ) = @_;
 
-   return $STRUCTS_BY_PERLNAME{$perlname} || croak "Unknown Tangence::Struct for '$perlname'";
+   return $STRUCTS_BY_PERLNAME{$perlname} or croak "Unknown Tangence::Struct for '$perlname'";
 }
 
 sub perlname
